@@ -11,23 +11,26 @@ import os
 import time
 import uuid
 from io import DEFAULT_BUFFER_SIZE
-from typing import Dict, List, Optional, Union
+from typing import Any, Dict, List, Optional, Union
 
 import fsspec
 import oci.data_science
+import oci.util as oci_util
 import yaml
-from oci.data_science.models import JobInfrastructureConfigurationDetails
-from oci.exceptions import ServiceError
+from ads.common import utils
 from ads.common.oci_datascience import DSCNotebookSession, OCIDataScienceMixin
 from ads.common.oci_logging import OCILog
 from ads.common.oci_resource import ResourceNotFoundError
-from ads.jobs.builders.runtimes.artifact import Artifact
 from ads.jobs.builders.infrastructure.base import Infrastructure, RunInstance
-from ads.jobs.builders.infrastructure.utils import get_value
 from ads.jobs.builders.infrastructure.dsc_job_runtime import (
     DataScienceJobRuntimeManager,
 )
-
+from ads.jobs.builders.infrastructure.utils import get_value
+from ads.jobs.builders.runtimes.artifact import Artifact
+from ads.jobs.builders.runtimes.container_runtime import ContainerRuntime
+from ads.jobs.builders.runtimes.python_runtime import GitPythonRuntime
+from oci.data_science.models import JobInfrastructureConfigurationDetails
+from oci.exceptions import ServiceError
 
 logger = logging.getLogger(__name__)
 
@@ -43,7 +46,7 @@ class DSCJob(OCIDataScienceMixin, oci.data_science.models.Job):
     .. code-block:: python
 
         job_properties = {
-            "display_name": "my_job,
+            "display_name": "my_job",
             "job_infrastructure_configuration_details": {"shape_name": "VM.MY_SHAPE"}
         }
         job = DSCJob(**job_properties)
@@ -69,13 +72,16 @@ class DSCJob(OCIDataScienceMixin, oci.data_science.models.Job):
             },
             "jobInfrastructureConfigurationDetails": {
                 "jobInfrastructureType": "STANDALONE",
-                "shapeName": "VM.Standard2.1",
+                "shapeName": "VM.Standard.E3.Flex",
+                "jobShapeConfigDetails": {
+                    "memoryInGBs": 16,
+                    "ocpus": 1
+                },
                 "blockStorageSizeInGBs": "100",
                 "subnetId": "<subnet_ocid>"
             }
         }
         job = DSCJob(**job_payload)
-
     """
 
     DEFAULT_INFRA_TYPE = (
@@ -129,12 +135,15 @@ class DSCJob(OCIDataScienceMixin, oci.data_science.models.Job):
 
     @artifact.setter
     def artifact(self, artifact: Union[str, Artifact]):
-        """Sets the job artifact"""
+        """Sets the job artifact."""
         self._artifact = artifact
 
     def _load_infra_from_notebook(self, nb_config):
         """Loads the infrastructure configuration from notebook configuration."""
         infra = self.job_infrastructure_configuration_details
+        nb_shape_config_details = oci_util.to_dict(
+            getattr(nb_config, "notebook_session_shape_config_details", None) or {}
+        )
         if isinstance(infra, dict):
             infra_type = infra.get("jobInfrastructureType", self.DEFAULT_INFRA_TYPE)
             shape_name = infra.get("shapeName", nb_config.shape)
@@ -142,6 +151,13 @@ class DSCJob(OCIDataScienceMixin, oci.data_science.models.Job):
                 "blockStorageSizeInGBs", nb_config.block_storage_size_in_gbs
             )
             subnet_id = infra.get("subnetId", nb_config.subnet_id)
+            job_shape_config_details = infra.get("jobShapeConfigDetails", {})
+            memory_in_gbs = job_shape_config_details.get(
+                "memoryInGBs", nb_shape_config_details.get("memory_in_gbs")
+            )
+            ocpus = job_shape_config_details.get(
+                "ocpus", nb_shape_config_details.get("ocpus")
+            )
         else:
             infra_type = (
                 infra.job_infrastructure_type
@@ -163,6 +179,15 @@ class DSCJob(OCIDataScienceMixin, oci.data_science.models.Job):
                 if getattr(infra, "subnet_id", None)
                 else nb_config.subnet_id
             )
+            job_shape_config_details = oci_util.to_dict(
+                getattr(infra, "job_shape_config_details", {}) or {}
+            )
+            memory_in_gbs = job_shape_config_details.get(
+                "memory_in_gbs", nb_shape_config_details.get("memory_in_gbs")
+            )
+            ocpus = job_shape_config_details.get(
+                "ocpus", nb_shape_config_details.get("ocpus")
+            )
 
         self.job_infrastructure_configuration_details = {
             "jobInfrastructureType": infra_type,
@@ -180,6 +205,17 @@ class DSCJob(OCIDataScienceMixin, oci.data_science.models.Job):
                 }
             )
 
+        # Specify shape config details
+        if memory_in_gbs or ocpus:
+            self.job_infrastructure_configuration_details.update(
+                {
+                    "jobShapeConfigDetails": {
+                        "memoryInGBs": memory_in_gbs,
+                        "ocpus": ocpus,
+                    }
+                }
+            )
+
     def load_properties_from_env(self) -> None:
         """Loads default properties from the environment"""
         if "NB_SESSION_OCID" in os.environ:
@@ -208,7 +244,7 @@ class DSCJob(OCIDataScienceMixin, oci.data_science.models.Job):
             self.to_oci_model(oci.data_science.models.CreateJobDetails)
         )
         self.update_from_oci_model(res.data)
-        if self.lifecycle_state == 'ACTIVE':
+        if self.lifecycle_state == "ACTIVE":
             return
         try:
             if issubclass(self.artifact.__class__, Artifact):
@@ -230,13 +266,15 @@ class DSCJob(OCIDataScienceMixin, oci.data_science.models.Job):
             The DSCJob instance (self), which allows chaining additional method.
 
         """
-        if not self.artifact:
-            raise ValueError("Artifact is required to create the job.")
         if not self.display_name:
-            timestamp = datetime.datetime.now().strftime("%Y%m%d-%H%M")
-            self.display_name = (
-                os.path.basename(str(self.artifact)).split(".")[0] + f"-{timestamp}"
-            )
+            if self.artifact:
+                timestamp = datetime.datetime.utcnow().strftime("%Y-%m-%d-%H:%M.%S")
+                self.display_name = (
+                    os.path.basename(str(self.artifact)).split(".")[0] + f"-{timestamp}"
+                )
+            else:
+                # Set default display_name if not specified - randomly generated easy to remember name generated
+                self.display_name = utils.get_random_name_for_resource()
         try:
             self.load_properties_from_env()
         except Exception as ex:
@@ -348,7 +386,7 @@ class DSCJob(OCIDataScienceMixin, oci.data_science.models.Job):
             * maximum_runtime_in_minutes: int
             * display_name: str
 
-        If display_name is not specified, it will be generated as "<JOB_NAME>-run-<TIMESTAMP>"
+        If display_name is not specified, it will be generated as "<JOB_NAME>-run-<TIMESTAMP>".
 
         Returns
         -------
@@ -368,18 +406,19 @@ class DSCJob(OCIDataScienceMixin, oci.data_science.models.Job):
             if key in swagger_types:
                 config_kwargs[key] = kwargs.pop(key)
 
-        if not kwargs.get("display_name"):
-            kwargs["display_name"] = (
-                self.display_name
-                + "-run-"
-                + datetime.datetime.now().strftime("%Y%m%d-%H%M")
-            )
+        # remove timestamp from the job name (added in default names, when display_name not specified by user)
+        if self.display_name:
+            try:
+                datetime.datetime.strptime(self.display_name[-19:], "%Y-%m-%d-%H:%M.%S")
+                self.display_name = self.display_name[:-20]
+            except ValueError:
+                pass
 
         job_attrs = dict(
             project_id=self.project_id,
             display_name=self.display_name
             + "-run-"
-            + datetime.datetime.now().strftime("%Y%m%d-%H%M"),
+            + datetime.datetime.now().strftime("%Y-%m-%d-%H:%M.%S"),
             job_id=self.id,
             compartment_id=self.compartment_id,
         )
@@ -561,7 +600,20 @@ class DataScienceJobRun(
         def stop_condition():
             """Stops the log streaming once the job is in a terminal state."""
             self.sync()
-            return self.lifecycle_state in self.TERMINAL_STATES
+            if self.lifecycle_state not in self.TERMINAL_STATES:
+                return False
+            # Stop if time_finished is not available.
+            if not self.time_finished:
+                return True
+            # Stop only if time_finished is over 1 minute ago.
+            # This is for the time delay between job run stopped and the logs appear in oci logging.
+            if (
+                datetime.datetime.now(self.time_finished.tzinfo)
+                - datetime.timedelta(minutes=1)
+                > self.time_finished
+            ):
+                return True
+            return False
 
         if not self.log_id and not self.log_group_id:
             print(
@@ -670,6 +722,9 @@ class DataScienceJob(Infrastructure):
     CONST_SHAPE_NAME = "shapeName"
     CONST_BLOCK_STORAGE = "blockStorageSize"
     CONST_SUBNET_ID = "subnetId"
+    CONST_SHAPE_CONFIG_DETAILS = "shapeConfigDetails"
+    CONST_MEMORY_IN_GBS = "memoryInGBs"
+    CONST_OCPUS = "ocpus"
     CONST_LOG_ID = "logId"
     CONST_LOG_GROUP_ID = "logGroupId"
 
@@ -682,10 +737,16 @@ class DataScienceJob(Infrastructure):
         CONST_SHAPE_NAME: "shape_name",
         CONST_BLOCK_STORAGE: "block_storage_size",
         CONST_SUBNET_ID: "subnet_id",
+        CONST_SHAPE_CONFIG_DETAILS: "shape_config_details",
         CONST_LOG_ID: "log_id",
         CONST_LOG_GROUP_ID: "log_group_id",
     }
 
+    shape_config_details_attribute_map = {
+        CONST_MEMORY_IN_GBS: "memory_in_gbs",
+        CONST_OCPUS: "ocpus",
+    }
+
     payload_attribute_map = {
         CONST_PROJECT_ID: "project_id",
         CONST_COMPARTMENT_ID: "compartment_id",
@@ -695,19 +756,35 @@ class DataScienceJob(Infrastructure):
         CONST_SHAPE_NAME: "job_infrastructure_configuration_details.shape_name",
         CONST_BLOCK_STORAGE: "job_infrastructure_configuration_details.block_storage_size_in_gbs",
         CONST_SUBNET_ID: "job_infrastructure_configuration_details.subnet_id",
+        CONST_SHAPE_CONFIG_DETAILS: "job_infrastructure_configuration_details.job_shape_config_details",
         CONST_LOG_ID: "job_log_configuration_details.log_id",
         CONST_LOG_GROUP_ID: "job_log_configuration_details.log_group_id",
     }
 
-    snake_to_camel_map = {v.split(".", maxsplit=1)[-1]: k for k, v in payload_attribute_map.items()}
+    snake_to_camel_map = {
+        v.split(".", maxsplit=1)[-1]: k for k, v in payload_attribute_map.items()
+    }
 
     @staticmethod
     def standardize_spec(spec):
         if not spec:
             return {}
+
+        attribute_map = {
+            **DataScienceJob.attribute_map,
+            **DataScienceJob.shape_config_details_attribute_map,
+        }
+        snake_to_camel_map = {v: k for k, v in attribute_map.items()}
+
         for key in list(spec.keys()):
-            if key not in DataScienceJob.payload_attribute_map and key in DataScienceJob.snake_to_camel_map:
-                spec[DataScienceJob.snake_to_camel_map[key]] = spec.pop(key)
+            if key not in attribute_map and key in snake_to_camel_map:
+                value = spec.pop(key)
+                if isinstance(value, dict):
+                    spec[snake_to_camel_map[key]] = DataScienceJob.standardize_spec(
+                        value
+                    )
+                else:
+                    spec[snake_to_camel_map[key]] = value
         return spec
 
     def __init__(self, spec: Dict = None, **kwargs) -> None:
@@ -919,6 +996,41 @@ class DataScienceJob(Infrastructure):
         """Subnet ID"""
         return self.get_spec(self.CONST_SUBNET_ID)
 
+    def with_shape_config_details(
+        self, memory_in_gbs: float, ocpus: float, **kwargs: Dict[str, Any]
+    ) -> DataScienceJob:
+        """Sets the details for the job run shape configuration.
+        Specify only when a flex shape is selected.
+        For example `VM.Standard.E3.Flex` allows the memory_in_gbs and cpu count to be specified.
+
+        Parameters
+        ----------
+        memory_in_gbs: float
+            The size of the memory in GBs.
+        ocpus: float
+            The OCPUs count.
+        kwargs
+            Additional keyword arguments.
+
+        Returns
+        -------
+        DataScienceJob
+            The DataScienceJob instance (self)
+        """
+        return self.set_spec(
+            self.CONST_SHAPE_CONFIG_DETAILS,
+            {
+                self.CONST_OCPUS: ocpus,
+                self.CONST_MEMORY_IN_GBS: memory_in_gbs,
+                **kwargs,
+            },
+        )
+
+    @property
+    def shape_config_details(self) -> Dict:
+        """The details for the job run shape configuration."""
+        return self.get_spec(self.CONST_SHAPE_CONFIG_DETAILS)
+
     def with_log_id(self, log_id: str) -> DataScienceJob:
         """Sets the log OCID for the data science job.
         If log ID is specified, setting the log group ID (with_log_group_id()) is not strictly needed.
@@ -1024,13 +1136,23 @@ class DataScienceJob(Infrastructure):
         -------
         DataScienceJob
             The DataScienceJob instance (self)
-
         """
+        sub_level = {
+            self.CONST_SHAPE_CONFIG_DETAILS: self.shape_config_details_attribute_map
+        }
         self.dsc_job = dsc_job
+
         for infra_attr, dsc_attr in self.payload_attribute_map.items():
             value = get_value(dsc_job, dsc_attr)
             if value:
-                self._spec[infra_attr] = get_value(dsc_job, dsc_attr)
+                if infra_attr not in sub_level:
+                    self._spec[infra_attr] = value
+                else:
+                    self._spec[infra_attr] = {}
+                    for sub_infra_attr, sub_dsc_attr in sub_level[infra_attr].items():
+                        sub_value = get_value(value, sub_dsc_attr)
+                        if sub_value:
+                            self._spec[infra_attr][sub_infra_attr] = sub_value
         return self
 
     def _update_job_infra(self, dsc_job: DSCJob) -> DataScienceJob:
@@ -1052,17 +1174,20 @@ class DataScienceJob(Infrastructure):
             self.CONST_SHAPE_NAME: "shapeName",
             self.CONST_SUBNET_ID: "subnetId",
             self.CONST_BLOCK_STORAGE: "blockStorageSizeInGBs",
+            self.CONST_SHAPE_CONFIG_DETAILS: "jobShapeConfigDetails",
         }
+
+        if not dsc_job.job_infrastructure_configuration_details:
+            dsc_job.job_infrastructure_configuration_details = {
+                self.CONST_JOB_INFRA: DSCJob.DEFAULT_INFRA_TYPE,
+                self.CONST_BLOCK_STORAGE: 50,
+            }
+
         for snake_attr, camel_attr in attr_map.items():
-            if self.get_spec(snake_attr):
-                if not dsc_job.job_infrastructure_configuration_details:
-                    dsc_job.job_infrastructure_configuration_details = {
-                        self.CONST_JOB_INFRA: DSCJob.DEFAULT_INFRA_TYPE,
-                        self.CONST_BLOCK_STORAGE: 50,
-                    }
-                dsc_job.job_infrastructure_configuration_details[
-                    camel_attr
-                ] = self.get_spec(snake_attr)
+            value = self.get_spec(snake_attr)
+            if value:
+                dsc_job.job_infrastructure_configuration_details[camel_attr] = value
+
         if dsc_job.job_infrastructure_configuration_details.get("subnetId"):
             dsc_job.job_infrastructure_configuration_details[
                 "jobInfrastructureType"
@@ -1090,7 +1215,13 @@ class DataScienceJob(Infrastructure):
         for attr in ["project_id", "compartment_id"]:
             if getattr(self, attr):
                 payload[attr] = getattr(self, attr)
-        payload["display_name"] = self.name
+        if isinstance(runtime, GitPythonRuntime) or isinstance(
+            runtime, ContainerRuntime
+        ):
+            payload["display_name"] = self.name or utils.get_random_name_for_resource()
+        else:
+            payload["display_name"] = self.name
+
         payload["job_log_configuration_details"] = self._prepare_log_config()
 
         self.dsc_job = DSCJob(**payload)
@@ -1109,7 +1240,7 @@ class DataScienceJob(Infrastructure):
         Parameters
         ----------
         name : str, optional
-            The name of the job run, by default None
+            The name of the job run, by default None.
         args : str, optional
             Command line arguments for the job run, by default None.
         env_var : dict, optional
